import datetime as dt
from typing import ClassVar, Optional, cast

from cuenca_validations.types import Country, PlatformRequest, State

from ..http import Session, session as global_session
from .base import Creatable


class Platform(Creatable):
    _resource: ClassVar = 'platforms'

    created_at: dt.datetime
    name: str
    rfc: Optional[str] = None
    establishment_date: Optional[dt.date] = None
    country: Optional[Country] = None
    state: Optional[State] = None
    economic_activity: Optional[str] = None
    email_address: Optional[str] = None
    phone_number: Optional[str] = None

    class Config:
        fields = {
            'name': {'description': 'name of the platform being created'},
            'rfc': {'description': 'RFC or CURP of the platform'},
            'establishment_date': {
                'description': 'when the platform was established'
            },
            'country': {'description': 'country where the platform resides'},
            'state': {'description': 'state where the platform resides'},
            'economic_activity': {'description': 'what the platform does'},
            'phone_number': {
                'description': 'phone number to contact the platform'
            },
            'email_address': {
                'description': 'email address to contact the platform'
            },
        }
        schema_extra = {
            'example': {
                'id': 'PT0123456789',
                'name': 'Arteria',
                'created_at': '2021-08-24T14:15:22Z',
                'rfc': 'ART123456FFF',
                'establishment_date': '2021-08-24T14:15:22Z',
                'country': 'MX',
                'state': 'DF',
                'economic_activity': 'fiinances and technologies',
                'phone_number': '+525555555555',
                'email_address': 'art@eria.com',
            }
        }

    @classmethod
    def create(
        cls,
        name: str,
        rfc: Optional[str] = None,
        establishment_date: Optional[str] = None,
        country: Optional[Country] = None,
        state: Optional[State] = None,
        economic_activity: Optional[str] = None,
        phone_number: Optional[str] = None,
        email_address: Optional[str] = None,
        *,
        session: Session = global_session,
    ) -> 'Platform':
        req = PlatformRequest(
            name=name,
            rfc=rfc,
            establishment_date=establishment_date,
            country=country,
            state=state,
            economic_activity=economic_activity,
            phone_number=phone_number,
            email_address=email_address,
        )
        return cast('Platform', cls._create(session=session, **req.dict()))



import pytest

from cuenca.resources import Platform


@pytest.mark.vcr
def test_platforms_create() -> None:
    platform = Platform.create(name='Test')
    assert platform.id
    assert platform.name == 'Test'



from typing import ClassVar

from .identity_events import IdentityEvent
from .users import User


class UserEvent(IdentityEvent):
    _resource: ClassVar = 'user_events'

    user_id: str
    platform_id: str

    class Config:
        schema_extra = {
            'example': {
                'id': 'UEYE4qnWs3Sm68tbgqkx_d5Q',
                'created_at': '2022-05-24T14:15:22Z',
                'identity_id': 'IDNEUInh69SuKXXmK95sROwQ',
                'type': 'created',
                'user_id': 'USWqY5cvkISJOxHyEKjAKf8w',
                'platform_id': 'PT8UEv02zBTcymd4Kd3MO6pg',
                'new_model': User.schema().get('example'),
            }
        }



import datetime as dt
from typing import ClassVar, Optional, cast

from cuenca_validations.types import (
    Country,
    CurpValidationRequest,
    Gender,
    State,
)
from cuenca_validations.types.identities import CurpField

from ..http import Session, session as global_session
from .base import Creatable, Retrievable


class CurpValidation(Creatable, Retrievable):
    _resource: ClassVar = 'curp_validations'

    created_at: dt.datetime
    names: Optional[str] = None
    first_surname: Optional[str] = None
    second_surname: Optional[str] = None
    date_of_birth: Optional[dt.date] = None
    country_of_birth: Optional[Country] = None
    state_of_birth: Optional[State] = None
    gender: Optional[Gender] = None
    nationality: Optional[Country] = None
    manual_curp: Optional[CurpField] = None
    calculated_curp: CurpField
    validated_curp: Optional[CurpField] = None
    renapo_curp_match: bool
    renapo_full_match: bool

    class Config:
        fields = {
            'names': {'description': 'Official name from Renapo'},
            'first_surname': {'description': 'Official surname from Renapo'},
            'second_surname': {'description': 'Official surname from Renapo'},
            'country_of_birth': {'description': 'In format ISO 3166 Alpha-2'},
            'state_of_birth': {'description': 'In format ISO 3166 Alpha-2'},
            'nationality': {'description': 'In format ISO 3166 Alpha-2'},
            'manual_curp': {'description': 'curp provided in request'},
            'calculated_curp': {
                'description': 'Calculated CURP based on request data'
            },
            'validated_curp': {
                'description': 'CURP validated in Renapo, null if not exists'
            },
            'renapo_curp_match': {
                'description': 'True if CURP exists and is valid'
            },
            'renapo_full_match': {
                'description': 'True if all fields provided match the response'
                ' from RENAPO. Accents in names are ignored'
            },
        }
        schema_extra = {
            'example': {
                'id': 'CVNEUInh69SuKXXmK95sROwQ',
                'created_at': '2019-08-24T14:15:22Z',
                'names': 'Guillermo',
                'first_surname': 'Gonzales',
                'second_surname': 'Camarena',
                'date_of_birth': '1965-04-18',
                'country_of_birth': 'MX',
                'state_of_birth': 'VZ',
                'gender': 'male',
                'nationality': 'MX',
                'manual_curp': None,
                'calculated_curp': 'GOCG650418HVZNML08',
                'validated_curp': 'GOCG650418HVZNML08',
                'renapo_curp_match': True,
                'renapo_full_match': True,
            }
        }

    @classmethod
    def create(
        cls,
        names: Optional[str] = None,
        first_surname: Optional[str] = None,
        date_of_birth: Optional[dt.date] = None,
        country_of_birth: Optional[str] = None,
        state_of_birth: Optional[State] = None,
        gender: Optional[Gender] = None,
        second_surname: Optional[str] = None,
        manual_curp: Optional[CurpField] = None,
        *,
        session: Session = global_session,
    ) -> 'CurpValidation':
        req = CurpValidationRequest(
            names=names,
            first_surname=first_surname,
            second_surname=second_surname,
            date_of_birth=date_of_birth,
            state_of_birth=state_of_birth,
            country_of_birth=country_of_birth,
            gender=gender,
            manual_curp=manual_curp,
        )
        return cast(
            'CurpValidation',
            cls._create(session=session, **req.dict()),
        )



import datetime as dt
from typing import ClassVar, List, Optional, cast

from clabe import Clabe
from cuenca_validations.types import (
    Address,
    AddressUpdateRequest,
    Beneficiary,
    KYCFile,
    KYCFileUpdateRequest,
    PhoneNumber,
    TOSAgreement,
    TOSUpdateRequest,
    UserQuery,
    UserRequest,
    UserStatus,
    UserUpdateRequest,
)
from cuenca_validations.types.identities import CurpField
from pydantic import EmailStr

from ..http import Session, session as global_session
from .balance_entries import BalanceEntry
from .base import Creatable, Queryable, Retrievable, Updateable
from .identities import Identity
from .resources import retrieve_uri


class User(Creatable, Retrievable, Updateable, Queryable):
    _resource: ClassVar = 'users'
    _query_params: ClassVar = UserQuery

    identity_uri: str
    level: int
    required_level: int
    created_at: dt.datetime
    phone_number: Optional[PhoneNumber]
    email_address: Optional[EmailStr]
    profession: Optional[str]
    terms_of_service: Optional[TOSAgreement]
    status: Optional[UserStatus]
    address: Optional[Address]
    govt_id: Optional[KYCFile]
    proof_of_address: Optional[KYCFile]
    proof_of_life: Optional[KYCFile]
    beneficiaries: Optional[List[Beneficiary]]
    platform_id: Optional[str] = None
    clabe: Optional[Clabe] = None

    @property
    def balance(self) -> int:
        be = cast(BalanceEntry, BalanceEntry.first(user_id=self.id))
        return be.rolling_balance if be else 0

    class Config:
        fields = {
            'level': {
                'description': 'Account level according to KYC information'
            },
            'required_level': {
                'description': 'Maximum level User can reach. Set by platform'
            },
            'govt_id': {
                'description': 'Detail of government id document validation'
            },
            'proof_of_address': {
                'description': 'Detail of proof of address document validation'
            },
            'proof_of_life': {
                'description': 'Detail of selfie video validation'
            },
            'beneficiaries': {
                'description': 'Beneficiaries of account in case of death'
            },
        }
        schema_extra = {
            'example': {
                'id': 'USWqY5cvkISJOxHyEKjAKf8w',
                'created_at': '2019-08-24T14:15:22Z',
                'updated_at': '2019-08-24T14:15:22Z',
                'identity_uri': 'identities/IDNEUInh69SuKXXmK95sROwQ',
                'level': 2,
                'required_level': 3,
                'phone_number': '+525511223344',
                'email_address': 'user@example.com',
                'profession': 'engineer',
                'terms_of_service': TOSAgreement.schema().get('example'),
                'status': 'active',
                'address': Address.schema().get('example'),
                'govt_id': KYCFile.schema().get('example'),
                'proof_of_address': None,
                'proof_of_life': None,
                'beneficiaries': [
                    Beneficiary.schema().get('example'),
                ],
                'platform_id': 'PT8UEv02zBTcymd4Kd3MO6pg',
            }
        }

    @classmethod
    def create(
        cls,
        curp: CurpField,
        phone_number: Optional[PhoneNumber] = None,
        email_address: Optional[EmailStr] = None,
        profession: Optional[str] = None,
        address: Optional[Address] = None,
        email_verification_id: Optional[str] = None,
        phone_verification_id: Optional[str] = None,
        status: Optional[UserStatus] = None,
        required_level: Optional[int] = None,
        *,
        session: Session = global_session,
    ) -> 'User':
        req = UserRequest(
            curp=curp,
            phone_number=phone_number,
            email_address=email_address,
            profession=profession,
            address=address,
            email_verification_id=email_verification_id,
            phone_verification_id=phone_verification_id,
            required_level=required_level,
            status=status,
        )
        return cast('User', cls._create(session=session, **req.dict()))

    @classmethod
    def update(
        cls,
        user_id: str,
        phone_number: Optional[PhoneNumber] = None,
        email_address: Optional[str] = None,
        profession: Optional[str] = None,
        address: Optional[AddressUpdateRequest] = None,
        beneficiaries: Optional[List[Beneficiary]] = None,
        govt_id: Optional[KYCFileUpdateRequest] = None,
        proof_of_address: Optional[KYCFileUpdateRequest] = None,
        proof_of_life: Optional[KYCFileUpdateRequest] = None,
        terms_of_service: Optional[TOSUpdateRequest] = None,
        verification_id: Optional[str] = None,
        email_verification_id: Optional[str] = None,
        phone_verification_id: Optional[str] = None,
        *,
        session: Session = global_session,
    ):
        request = UserUpdateRequest(
            phone_number=phone_number,
            email_address=email_address,
            profession=profession,
            address=address,
            beneficiaries=beneficiaries,
            govt_id=govt_id,
            proof_of_address=proof_of_address,
            proof_of_life=proof_of_life,
            terms_of_service=terms_of_service,
            verification_id=verification_id,
            email_verification_id=email_verification_id,
            phone_verification_id=phone_verification_id,
        )
        return cast(
            'User',
            cls._update(id=user_id, **request.dict(), session=session),
        )

    @property
    def identity(self) -> Identity:
        return cast(Identity, retrieve_uri(self.identity_uri))
